/*
 * Copyright(c) 2021-2023 Analog Devices, Inc.All Rights Reserved.
 * This software is proprietary to Analog Devices, Inc. and its licensors.
 */

/** @file adrv9002_iio.h
 * @brief Public interface */

#ifndef __ADRV9002_IIO_H__
#define __ADRV9002_IIO_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h>
#include <stdint.h>
#include <stddef.h>
#ifdef ENABLE_IIO
#include <iio.h>
#endif

#ifdef _WIN32
#ifdef LIBADRV9002_EXPORTS
#define __api __declspec(dllexport)
#else
#define __api __declspec(dllimport)
#endif
#elif __GNUC__ >= 4 && !defined(MATLAB_MEX_FILE) && !defined(MATLAB_LOADLIBRARY)
#define __api __attribute__((visibility("default")))
#else
#define __api
#endif

#ifdef ENABLE_IIO
struct iio_device;
#endif

// Known missing features
// 1. TX FM modes
// 2. External LO
// 3. PLL retuning
// 4. Strobe types are not independent


/**
 * @struct rx_radio_channel_config
 * @brief RX channel configuration
 * 
 * A structure containing the configuration for a single RX channel
 */
typedef struct rx_radio_channel_config {
  /** Enable channel */
  bool enabled;
  /** Enable high performance ADC, otherwise use low-power ADC */
  bool adc_high_performance_mode;
  /** Enable ADC frequency offset correction */
  bool frequency_offset_correction_enable;
  /** Power mode of front-end analog filter Options are:
  0 - Low power
  1 - Medium power
  2 - High power
  */
  uint8_t analog_filter_power_mode;
  /** Use second order (Biquad) analog filter, otherwise first order TIA is used */
  bool analog_filter_biquad;
  /** Front-end analog filter 1dB (Biquad) or 3 dB (TIA) bandwidth in Hz*/
  uint32_t analog_filter_bandwidth_hz;
  /** Channel bandwidth of interest at ADC in Hz*/
  uint32_t channel_bandwidth_hz;
  /** RX channel sample rate at digital interface */
  uint32_t sample_rate_hz;
  /** Enable NCO to perform frequency translation */
  bool nco_enable;
  /** NCO frequency in Hz */
  int32_t nco_frequency_hz;
  /** RF port source used for channel Options are:
  0 - RX_A
  1 - RX_B
  */
  uint8_t rf_port;

} rx_radio_channel_config;

/**
 * @struct tx_radio_channel_config
 * @brief TX channel configuration
 * 
 * A structure containing the configuration for a single TX channel
 */
typedef struct tx_radio_channel_config {
  /** Enable channel */
  bool enabled;
  /** Data rate at digital interface in Hz */
  uint32_t sample_rate_hz;
  /** Enable DAC frequency offset correction */
  bool frequency_offset_correction_enable;
  /** Power mode of front-end analog filter Options are:
  0 - Low power
  1 - Medium power
  2 - High power
  */
  uint8_t analog_filter_power_mode;
  /** Channel bandwidth of interest at DAC in Hz*/
  uint32_t channel_bandwidth_hz;
  /** Enable observation path */
  bool orx_enabled;
  /** Set external loopback mode. Options are:
  0 - Disabled
  1 - Before PA
  2 - After PA
  */
  uint8_t elb_type;
} tx_radio_channel_config;

/**
 * @struct radio_config
 * @brief Device configuration
 * 
 * A structure containing the configuration for the top-level device
 */
typedef struct radio_config {
  /** SSI lanes to use Valid cases:
    1 (CMOS/LVDS)
    2 (LVDS)
    4 (CMOS)
  */
  uint8_t ssi_lanes;
  /** Use DDR mode at digital interface, false will use SDR */
  bool ddr;
  /** Use short strobe mode at digital interface, false will use long strobe */
  bool short_strobe;
  /** Use LVDS mode at digital interface, false will use CMOS*/
  bool lvds;
  /** ADC clock rate mode select. Options are:
  1 = LOW
  2 = MEDIUM
  3 = HIGH
  */
  uint8_t adc_rate_mode;
  /** Use FDD duplex mode, false will use TDD */
  bool fdd;
  /** Channel configurations for RX1 and RX2 */
  rx_radio_channel_config rx_config[2];
  /** Channel configurations for TX1 and TX2 */
  tx_radio_channel_config tx_config[2];

} radio_config;

/**
 * @struct clock_config
 * @brief Clock configuration
 * 
 * A structure containing the configuration for the device clock
 */
typedef struct clock_config {
  /** Device clock frequency in kHz */
  uint32_t device_clock_frequency_khz;
  bool device_clock_output_enable;
  uint8_t device_clock_output_divider;
  /** Enable high performance PLL mode, otherwise low-power mode is used*/
  bool clock_pll_high_performance_enable;
  /** PLL power mode. Options:
     0 = low power
     1 = medium performance
     2 = high performance
  */
  uint8_t clock_pll_power_mode;
  /** Processor clock divider. Valid values are 1, 2, 4, 8, 16, 32, 64, 128, 256
  */
  uint8_t processor_clock_divider;
} clock_config;


/**
 * @struct adrv9002_config
 * @brief Top-level configuration
 * 
 * A structure containing the configuration for the top-level device
 */
typedef struct adrv9002_config {
  radio_config radio_cfg;
  clock_config clk_cfg;
} adrv9002_config;

/*
---------------------------------------------------------------------------*/
/* ------------------------- Top-level functions
-----------------------------*/
/** @defgroup TopLevel Top-level functions
 * @{ */

/**
 * @brief Generate profile from configuration
 * 
 * @param cfg Nested struct of device configuration for shared and individual channels
 * @param profile Resulting json profile
 * @param profile_num_bytes Number of bytes in profile array
 * @param stream_image Stream image for ARM stream processor
 * @param stream_image_num_bytes Number of bytes in stream image array
 * @return On success, 0 is returned
 * @return On error, a negative errno code is returned
 */
__api int adrv9002_generate_profile(adrv9002_config *cfg, char **profile,
                                    size_t *profile_num_bytes,
                                    char **stream_image, size_t *stream_image_num_bytes);


/**
 * @brief Generate profile from configuration and load it to device with IIO driver
 * 
 * @param ac Nested struct of device configuration for shared and individual channels
 * @param dev A pointer to the iio_device structure of the adrv9002-phy
 * @return On success, 0 is returned
 * @return On error, a negative errno code is returned
 */
#ifdef ENABLE_IIO
__api int adrv9002_generate_load_profile_iio(adrv9002_config *ac, struct iio_device *dev);
#endif

/**
 * @brief Get error message from error codes. This also includes profile specific codes
 *
 * @param error_code Error code to lookup and return message for
 * @return On code found, message is returned
 * @return On code not found, an empty string is returned
 */
__api char *get_message_from_error_code(int error_code);

/**
 * @brief Get the api version
 *
 * @return profile gen api version as a string
 */
__api const char *get_api_version();


/**
 * @brief Get the interface mode of the device
 *
 * @param dev A pointer to the iio_device structure of the adrv9002-phy
 * @param mode 4 character string containing the interface mode. (Malloc before passing)
 * @return On success, 0 is returned
 * @return On error, a negative errno code is returned
 */
#ifdef ENABLE_IIO
__api int adrv9002_get_interface_mode_iio(struct iio_device *dev, char *mode);
#endif

/** @} */

#ifdef __cplusplus
}
#endif

#undef __api

#endif /* __ADRV9002_IIO_H__ */